close all; clearvars; clc;

seed = 5;
rng(seed);

N = 50;
thr = 1e-5;
nSamples = 2000;
sigma = 0.01;
[A1,~, ~] = construct_graph(N,'er',0.2);
A1 = full(A1);

p_change = 0.10;
vec_A1 = my_upper(A1);
for j=1:length(vec_A1)
    if vec_A1(j,1)~=0
        temp_vec(j,1) = rand;
    else
        temp_vec(j,1) = 0;
    end
end
temp_vec(temp_vec<p_change) = 0;
temp_vec(temp_vec~=0) = 1;
[temp_A,~, ~] = construct_graph(N,'er',0.2);
temp_vec2 = my_upper(full(temp_A));
for j=1:length(temp_vec2)
    if temp_vec2(j,1)~=0
        temp_vec2(j,1) = rand;
    else
        temp_vec2(j,1) = 0;
    end
end
temp_vec2(temp_vec2>(1-p_change)) = 1;
temp_vec2(temp_vec2~=1) = 0;
new_vec = temp_vec + temp_vec2;
new_vec(new_vec~=0) = 1;
A2 = squareform(new_vec, 'tomatrix');

L = diag(sum(A1,2)) - A1;
[V,D] = eig(L);
d = pinv(D);
mu = zeros(1,N);
gftcoeff = mvnrnd(mu,d,nSamples/2);
X = V*gftcoeff';
X_noisy1 = X + sigma*randn(size(X));

L = diag(sum(A2,2)) - A2;
[V,D] = eig(L);
d = pinv(D);
mu = zeros(1,N);
gftcoeff = mvnrnd(mu,d,nSamples/2);
X = V*gftcoeff';
X_noisy2 = X + sigma*randn(size(X));

X_noisy = cat(2,X_noisy1,X_noisy2);

%%
%==========================================================================
% grid search
%==========================================================================
fprintf('Begining grid search... \n');

Z = sparse(gsp_distanz(X_noisy1').^2);
a = 10.^[1:-0.2:-1];
b = 10.^[1:-0.2:-1];

idx = 1;
for i=1:length(a)
    for j=1:length(b)
        param(:,idx) = [a(i);b(j)];
        idx = idx + 1;
    end
end

for t=1:size(param,2)
    [w_test,~] = gsp_learn_graph_log_degrees(Z/(nSamples/2), param(1,t), param(2,t));
    w_test(w_test<thr) = 0;
    [~,~,F(t),~,~] = graph_learning_perf_eval(A1,w_test);
end
idx = find(F==max(F));
temp_b = param(2,idx);
b_id = find(temp_b==max(temp_b));
idx = idx(b_id);
idx = idx(1);
F_best = max(F);
a = param(1,idx);
b = param(2,idx);

fprintf('The grid search is finished! \n');

params.tol = 1e-12;
params.maxit = 10000;
params.step_size = 0.2;

Z_kal = sparse(gsp_distanz(X_noisy1').^2);
[W_kal, ~] = gsp_learn_graph_log_degrees(Z_kal/(nSamples/2), a, b, params);
W_kal(W_kal<thr) = 0;
temp = squareform_sp(W_kal)';
w_optim1 = temp(:);

Z_kal = sparse(gsp_distanz(X_noisy2').^2);
[W_kal, ~] = gsp_learn_graph_log_degrees(Z_kal/(nSamples/2), a, b, params);
W_kal(W_kal<thr) = 0;
temp = squareform_sp(W_kal)';
w_optim2 = temp(:);


%%
%==========================================================================
% Calculating the error
%==========================================================================
fprintf('Begining error calculation... \n');

k = 0.002;

[S, St] = sum_squareform(N);
L = 1.0*((N-1)/b);
prox_g = @(x) (x+sqrt(x.^2+4*a*L))./2;
yk = zeros(N,1);


Z = sparse(gsp_distanz(X_noisy(:,1)').^2);
z_old = squareform_sp(Z);


w = zeros(N*(N-1)/2,1);
f.prox = @(w,z,c) max(eps, w - 2.*c.*z);
norm_S = sqrt(2*(N-1));
g.grad = @(w) 2 * b * w - a * St * (1./(S*w));
g.beta = @(w) 2 * b + (a* norm_S^2)/(min(S*w)^2);

iniTime = clock;
for itr=1:nSamples/2
    Z = sparse(gsp_distanz(X_noisy(:,itr)').^2);
    z = squareform_sp(Z);
    z_new = z .* k + z_old .* (1-k);    
    
    u = max(0,(St*yk-2*z_new)./(2*b));
    v = prox_g((S*u-L*yk));
    yK = yk - (1/L).*(S*u-v);
    yk = yK;
    w_hat = max(0,(St*yK-2*z_new)./(2*b));
    w_hat(w_hat<thr) = 0;
    err_DPG(itr) = norm(w_hat - w_optim1);
    
    
    step = 1.0 * (1/g.beta(w));
    w_k = f.prox(w - step * g.grad(w),z_new,step);
    w = w_k;
    w_hat_PG = w;
    w_hat_PG(w_hat_PG<thr) = 0;
    err_PG(itr) = norm(w_hat_PG - w_optim1);
    
    z_old = z_new;
    fprintf('Iteration %d completed! Elapsed time: %.2f sec.\n', itr, etime(clock, iniTime));
end


for itr=1:nSamples/2
    Z = sparse(gsp_distanz(X_noisy(:,(nSamples/2)+itr)').^2);
    z = squareform_sp(Z);
    z_new = z .* k + z_old .* (1-k);
    
    
    u = max(0,(St*yk-2*z_new)./(2*b));
    v = prox_g((S*u-L*yk));
    yK = yk - (1/L).*(S*u-v);
    yk = yK;
    w_hat = max(0,(St*yK-2*z_new)./(2*b));
    w_hat(w_hat<thr) = 0;
    err_DPG((nSamples/2)+itr) = norm(w_hat - w_optim2);
    
    
    step = 1.0 * (1/g.beta(w));
    w_k = f.prox(w - step * g.grad(w),z_new,step);
    w = w_k;
    w_hat_PG = w;
    w_hat_PG(w_hat_PG<thr) = 0;
    err_PG((nSamples/2)+itr) = norm(w_hat_PG - w_optim2);
    
    z_old = z_new;
    fprintf('Iteration %d completed! Elapsed time: %.2f sec.\n', (nSamples/2)+itr, etime(clock, iniTime));
end

fprintf('Done! Wait for the plot!\n');


%==========================================================================
% Error figure
%==========================================================================

h1 = figure(1);
hold on
p1 = plot(err_PG, '-.', 'LineWidth',1.0,'Color','#C3073F');
hold on
p2 = plot(err_DPG, '-', 'LineWidth',1.0,'Color','#0f4d92');
ticks = 250:250:nSamples;

grid on
ax = gca;
ax.GridLineStyle = '--';
ax.GridAlpha = 0.75;
legend([p1 p2], {'online PG','online DPG'},'Location','northeast')
xlabel('Number of iterations')
xlim([100 nSamples])
xticks(ticks)
ylabel('$\| w - w^{\star}\|_{2}$','Interpreter','Latex')
set(gca, 'YScale','log')