close all; clearvars; clc;

seed = 0;
rng(seed);

N = 76;
thr = 1e-3;
sub_id = 1;
subjects = dir('.\epilepsy\*_pre.dat');
subjects = {subjects.name};
X1 = load(subjects{sub_id});

subjects = dir('.\epilepsy\*_ict.dat');
subjects = {subjects.name};
X2 = load(subjects{sub_id});

X_noisy = cat(2,X1,X2);


%%
%==========================================================================
% Calculating the error
%==========================================================================
fprintf('Begining error calculation... \n');

k = 0.002;
a = 0.1;
b = 1.0;

[S, St] = sum_squareform(N);
L = 1.0*((N-1)/b);
prox_g = @(x) (x+sqrt(x.^2+4*a*L))./2;
yk = zeros(N,1);


Z = sparse(gsp_distanz(X_noisy(:,1)').^2);
z_old = squareform_sp(Z) / 1e5;

iniTime = clock;
for itr=1:8000
    Z = sparse(gsp_distanz(X_noisy(:,itr)').^2);
    z = squareform_sp(Z) / 1e5;
    z_new = z .* k + z_old .* (1-k);
    
    
    u = max(0,(St*yk-2*z_new)./(2*b));
    v = prox_g((S*u-L*yk));
    yK = yk - (1/L).*(S*u-v);
    yk = yK;
    w_hat = max(0,(St*yK-2*z_new)./(2*b));
    w_hat(w_hat<thr) = 0;
    w_DPG(:,itr) = w_hat;
    if itr==3000
        W1_DPG = squareform(w_hat);
        W1_DPG(W1_DPG>0) = 1;
    end
    if itr==5000
        W2_DPG = squareform(w_hat);
        W2_DPG(W2_DPG>0) = 1;
    end
    
    z_old = z_new;
    fprintf('Iteration %d completed! Elapsed time: %.2f sec.\n', itr, etime(clock, iniTime));
end

fprintf('Done! Wait for the plot!\n');

%%
%==========================================================================
% figures
%==========================================================================

xcord = [8, 7, 6, 5, 4, 3, 2, 1, 9, 8, 7, 6, 5, 4, 3, 2, 10, 9, 8, 7, 6, 5, 4,...
         3, 11, 10, 9, 8, 7, 6, 5, 4, 12, 11, 10, 9, 8, 7, 6, 5, 13, 12, 11, 10,...
         9, 8, 7, 6, 14, 13, 12, 11, 10, 9, 8, 7, 15, 14, 13, 12, 11, 10, 9, 8,...
         0, 0, 0, 0, 0, 0, 2, 3, 4, 5, 6, 7];
ycord = [1, 2, 3, 4, 5, 6, 7, 8, 2, 3, 4, 5, 6, 7, 8, 9, 3, 4, 5, 6, 7, 8, 9,...
         10, 4, 5, 6, 7, 8, 9, 10, 11, 5, 6, 7, 8, 9, 10, 11, 12, 6, 7, 8, 9, 10,...
         11, 12, 13, 7, 8, 9, 10, 11, 12, 13, 14, 8, 9, 10, 11, 12, 13, 14, 15,...
         2, 3, 4, 5, 6,7, 0, 0, 0, 0, 0, 0];
     
G1 = graph(W1_DPG);
G2 = graph(W2_DPG);
C1 = centrality(G1, 'closeness');
C2 = centrality(G2, 'closeness');


h1 = figure(1);
set(gcf,'position',[300,300,600,500]);
gr1 = plot(G1);
gr1.XData = xcord;
gr1.YData = ycord;
gr1.MarkerSize = 5;
G1.Nodes.value = C1;
G1.Nodes.NodeColors = G1.Nodes.value;
gr1.NodeCData = G1.Nodes.NodeColors;
gr1.LineWidth = 1;
gr1.EdgeAlpha = 0.20;
gr1.NodeLabel = {};
xlim([-1 17]);
ylim([-1 17]);
caxis([min(min(C1,C2)), max(max(C1,C2))]);
set(gca,'XTick',[], 'YTick', [])

h2 = figure(2);
set(gcf,'position',[300,300,600,500]);
gr2 = plot(G2);
gr2.XData = xcord;
gr2.YData = ycord;
gr2.MarkerSize = 5;
G2.Nodes.value = C2;
G2.Nodes.NodeColors = G2.Nodes.value;
gr2.NodeCData = G2.Nodes.NodeColors;
gr2.LineWidth = 1;
gr2.EdgeAlpha = 0.20;
gr2.NodeLabel = {};
xlim([-1 17]);
ylim([-1 17]);
caxis([min(min(C1,C2)), max(max(C1,C2))]);
set(gca,'XTick',[], 'YTick', [])

h3 = figure(3);
p3 = imshow(w_DPG);
axis square
caxis([min(w_DPG(:)), max(w_DPG(:))]);
xlabel('Time samples')
ylabel('All possible edges')
axis on
xticks([1000:1000:8000])
xline(4000, '-.', 'LineWidth', 2, 'Color', 'k');
colorbar;
set(gca,'ytick',[])